package gov.va.med.pharmacy.peps.external.common.fdbcloud.capability.impl;

import gov.va.med.pharmacy.peps.common.exception.DataNotFoundException;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckVo;
import gov.va.med.pharmacy.peps.common.vo.DrugDrugCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.DrugTherapyCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckVo;
import gov.va.med.pharmacy.peps.common.vo.ProfessionalMonographVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.PerformDrugChecksCapability;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.utility.StringUtility;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.capability.GetProfessionalMonographCapability;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.capability.ScreenCapability;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.capability.utility.FDBCloudUtility;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.CallContext;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DDIReferenceSection;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DDIScreenRequest;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DDIScreenResponse;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DDIScreenResult;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DPTClass;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DPTScreenRequest;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DPTScreenResponse;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DPTScreenResult;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DrugConceptType;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.DrugDrugMonograph;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.Note;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.Screen;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.ScreenDrug;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.ScreenProfile;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.ScreenResponse;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.ScreenResult;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.Section;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

/**
 * Implementation of PerformDrugChecksCapability that uses FDB Cloud Connector
 * REST API. Supports Order Checks for Drug Drug Interaction and Duplicate
 * Therapy. Dosing Checks are not supported, since FDB Cloud Connector does not
 * support it at the time this class was coded.
 */
public class PerformDrugChecksCapabilityImpl implements PerformDrugChecksCapability {

    /** The Constant LOG. */
    private static final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
            .getLogger(PerformDrugChecksCapabilityImpl.class);

    /** The Constant SEVERITY_FILTER_INCLUDE_ALL. */
    private static final int SEVERITY_FILTER_INCLUDE_ALL = 9;

    /** The get professional monograph capability. */
    private GetProfessionalMonographCapability getProfessionalMonographCapability;

    /** The screen capability. */
    private ScreenCapability screenCapability;

    /**
     * Convert the OrderCheckVo into FDB Cloud Connector Requests, perform the
     * checks, and convert the FDB Cloud Connector response to an
     * OrderCheckResultsVo.
     *
     * @param requestVo OrderCheckVo requesting checks
     * @return OrderCheckResultsVo with results from FDB Cloud Connector
     */
    @Override
    public final OrderCheckResultsVo performDrugChecks(OrderCheckVo requestVo) {

        OrderCheckResultsVo results = new OrderCheckResultsVo();

        CallContext callContext = FDBCloudUtility.createCallContext();

        List<ScreenDrug> screenDrugs = new ArrayList<ScreenDrug>();
        Map<String, DrugCheckVo> drugMap = new HashMap<String, DrugCheckVo>();
        int uniqueId = 0;

        for (DrugCheckVo drug : requestVo.getDrugsToScreen()) {
            ScreenDrug screenDrug = toScreenDrug(drug);
            // To be sure to make all drugs unique as GCN sequence number, IEN,
            // and order number are not unique, use an internal incremented
            // unique ID.
            screenDrug.setTag(String.valueOf(uniqueId));
            drugMap.put(screenDrug.getTag(), drug);
            screenDrugs.add(screenDrug);
            uniqueId++;

        }

        ScreenProfile screenProfile = new ScreenProfile();
        screenProfile.setScreenDrugs(screenDrugs);

        Screen screen = new Screen();
        screen.setCallContext(callContext);
        screen.setScreenProfile(screenProfile);

        if (requestVo.isDrugDrugCheck()) {
            DDIScreenRequest ddiScreenRequest = new DDIScreenRequest();
            ddiScreenRequest.setSeverityFilter(SEVERITY_FILTER_INCLUDE_ALL);
            ddiScreenRequest.setProspectiveOnly(requestVo.isProspectiveOnly());
            screen.setDdiScreenRequest(ddiScreenRequest);

        }

        if (requestVo.isDrugTherapyCheck()) {
            DPTScreenRequest dptScreenRequest = new DPTScreenRequest();
            dptScreenRequest.setUseDuplicateAllowance(requestVo.isDuplicateAllowance());
            dptScreenRequest.setProspectiveOnly(requestVo.isProspectiveOnly());
            screen.setDptScreenRequest(dptScreenRequest);
        }

        try {
            ScreenResult screenResult = getScreenCapability().screen(screen);
            if (!screenResult.getIsSuccessful()) {
                LOG.warn("Screen Result not successful.");
            }

            results.setHeader(requestVo.getHeader());

            if (requestVo.isDrugDrugCheck()) {
                results.setDrugDrugCheckResults(convertDDIScreenResponse(screenResult.getDDIScreenResponse(),
                        screenDrugs, drugMap));
                convertNotes(screenResult.getDDIScreenResponse(), results, drugMap);
            }

            if (requestVo.isDrugTherapyCheck()) {
                results.setDrugTherapyCheckResults(convertDPTScreenResponse(screenResult.getDPTScreenResponse(),
                        screenDrugs, drugMap));
                convertNotes(screenResult.getDPTScreenResponse(), results, drugMap);
            }

        } catch (DataNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return results;
    }

    /**
     * Convert notes.
     *
     * @param screenResponse the screen response
     * @param results the results
     * @param drugMap the drug map
     */
    private void convertNotes(ScreenResponse screenResponse, OrderCheckResultsVo results,
            Map<String, DrugCheckVo> drugMap) {
        if (screenResponse.getNotes() == null) {
            return;
        }

        for (Note note : screenResponse.getNotes()) {
            if (Note.NOTE_TYPE_NOT_FOUND == note.getNoteType()) {
                if (results.getDrugsNotChecked() == null) {
                    results.setDrugsNotChecked(new ArrayList<DrugCheckVo>());
                }
                LOG.warn("Drug with GCN Sequence Number " + note.getItemID() + " was not checked");
                // Note doesn't have the tag, so we have to find the original
                // DrugCheckVo the hard way
                for (DrugCheckVo drugCheckVo : drugMap.values()) {
                    if (note.getItemID().equals(drugCheckVo.getGcnSeqNo())) {
                        results.getDrugsNotChecked().add(drugCheckVo);
                        break;
                    } else {
                        LOG.warn("Could not map Note item id " + note.getItemID());
                    }
                }
            }
        }

    }

    /**
     * To screen drug.
     *
     * @param drug the drug
     * @return the screen drug
     */
    private ScreenDrug toScreenDrug(DrugCheckVo drug) {
        ScreenDrug screenDrug = new ScreenDrug();
        screenDrug.setDrugConceptType(DrugConceptType.DISPENSABLE_GENERIC.getId()); // DispensibleGeneric
        if (StringUtils.isNotBlank(drug.getDrugName())) {
            screenDrug.setDrugDesc(drug.getDrugName());
        }
        screenDrug.setDrugID(drug.getGcnSeqNo());
        screenDrug.setProspective(drug.isProspective());
        return screenDrug;
    }

    /**
     * Convert from the results received into Collection of value objects.
     *
     * @param screenResponse results from FDB
     * @param screenDrugs FDB ScreenDrugs used as input to dose check call
     * @param drugMap Map of the combined String of IEN and order number to
     *            DrugCheckVo. The combined String of IEN and order number is
     *            stored in the FDB ScreenDrug's custom ID attribute.
     * @return DrugCheckResultsVO results converted from Drug Data Vendor return
     *         containing DrugDrugCheckResultVO for checks
     */
    private DrugCheckResultsVo<DrugDrugCheckResultVo> convertDDIScreenResponse(DDIScreenResponse screenResponse,
            List<ScreenDrug> screenDrugs, Map<String, DrugCheckVo> drugMap) {
        if (screenResponse == null) {
            return null;
        }
        DrugCheckResultsVo<DrugDrugCheckResultVo> vo = new DrugCheckResultsVo<DrugDrugCheckResultVo>();

        String[] severityArray = null;
        
        for (DDIScreenResult result : screenResponse.getDDIScreenResults()) {
            DrugDrugCheckResultVo check = new DrugDrugCheckResultVo();

            check.setId(result.getInteractionID());
            check.setDrugs(new ArrayList<DrugCheckVo>(2));

            DrugCheckVo drug1 = drugMap.get(result.getScreenDrug1().getTag());
            check.getDrugs().add(drug1);
            DrugCheckVo drug2 = drugMap.get(result.getScreenDrug2().getTag());
            check.getDrugs().add(drug2);

            check.setInteractionDescription(StringUtility.nullToEmptyString(result.getInteractionDesc()));
            check.setShortText(StringUtility.nullToEmptyString(result.getScreenMessage()));

            //remove extra text from severity
            if(StringUtils.isNotEmpty(result.getSeverityDesc()) && result.getSeverityDesc().contains(":")){
                severityArray =  result.getSeverityDesc().split(":");
                check.setSeverity(severityArray[0]);
            }
            else{
                check.setSeverity(StringUtility.nullToEmptyString(result.getSeverityDesc()));
            }
            

            check.setProfessionalMonograph(getProfessionalMonograph(result.getMonographID()));

            vo.getChecks().add(check);
        }
        return vo;
    }

    /**
     * Gets the professional monograph.
     *
     * @param monographID the monograph id
     * @return the professional monograph
     */
    private ProfessionalMonographVo getProfessionalMonograph(Integer monographID) {
        if (monographID == null) {
            return null;
        }
        DrugDrugMonograph drugdrugMonograph = getGetProfessionalMonographCapability().getProfessionalMonograph(
                monographID);

        ProfessionalMonographVo pm = transformFDBDrugDrugMonographToProfessionalMonographVo(drugdrugMonograph);
        return pm;
    }

    /**
     * Transform fdb drug drug monograph to professional monograph vo.
     *
     * @param drugdrugMonograph the drugdrug monograph
     * @return ProfessionalMonographVo
     */
    private ProfessionalMonographVo transformFDBDrugDrugMonographToProfessionalMonographVo(
            DrugDrugMonograph drugdrugMonograph) {
        List<Section> sections = drugdrugMonograph.getSections();
        List<DDIReferenceSection> referenceSections = drugdrugMonograph.getReferenceSections();

        ProfessionalMonographVo pm = new ProfessionalMonographVo();
        // no custom in FDB Cloud, so we only have FDB source
        pm.setFdbMonographSourceType(true);

        Map<String, Section> sectionsMap = new HashMap<String, Section>();
        for (Section section : sections) {
            sectionsMap.put(section.getSectionCode(), section);
            LOG.debug(section.toString());
        }

        pm.setMonographTitle(getMonographSectionText(sectionsMap, ProfessionalMonographVo.TITLE_SECTION));
        pm.setSeverityLevel(getMonographSectionText(sectionsMap, ProfessionalMonographVo.SEVERITY_SECTION));
        pm.setMechanismOfAction(getMonographSectionText(sectionsMap, ProfessionalMonographVo.MECHANISM_SECTION));
        pm.setClinicalEffects(getMonographSectionText(sectionsMap, ProfessionalMonographVo.CLINICAL_SECTION));
        pm.setPatientManagement(getMonographSectionText(sectionsMap, ProfessionalMonographVo.PATIENT_SECTION));
        pm.setPredisposingFactors(getMonographSectionText(sectionsMap, ProfessionalMonographVo.PREDISPOSING_SECTION));
        pm.setDiscussion(getMonographSectionText(sectionsMap, ProfessionalMonographVo.DISCUSSION_SECTION));
        pm.setReferences(getReferenceList(referenceSections));

        return pm;
    }

    /**
     * Gets the reference list.
     *
     * @param referenceSections the reference sections
     * @return the reference list
     */
    private List<String> getReferenceList(List<DDIReferenceSection> referenceSections) {
        List<String> referenceList = new ArrayList<String>();
        referenceList.add("REFERENCES:");

        for (DDIReferenceSection ddiReferenceSection : referenceSections) {
            StringBuffer buffer = new StringBuffer();
            buffer.append(ddiReferenceSection.getReferenceSequence());
            buffer.append(".");
            buffer.append(ddiReferenceSection.getReferenceText());
            referenceList.add(buffer.toString());
        }
        return referenceList;
    }

    /**
     * Gets the monograph section text.
     *
     * @param sectionsMap the sections map
     * @param sectionCode the section code
     * @return the monograph section text
     */
    private String getMonographSectionText(Map<String, Section> sectionsMap, String sectionCode) {
        if (sectionsMap.containsKey(sectionCode)) {
            return (sectionsMap.get(sectionCode).getTitle().toUpperCase() + ":  " + sectionsMap.get(sectionCode)
                    .getText());

        }
        return null;
    }

    /**
     * Convert from the results received into Collection of value objects.
     *
     * @param screenResponse results from FDB
     * @param screenDrugs FDB ScreenDrugs used as input to dose check call
     * @param drugMap Map of the combined String of IEN and order number to
     *            DrugCheckVo. The combined String of IEN and order number is
     *            stored in the FDB ScreenDrug's custom ID attribute.
     * @return DrugCheckResultsVO results converted from Drug Data Vendor return
     *         containing DrugDrugCheckResultVO for checks
     */
    private DrugCheckResultsVo<DrugTherapyCheckResultVo> convertDPTScreenResponse(DPTScreenResponse screenResponse,
            List<ScreenDrug> screenDrugs, Map<String, DrugCheckVo> drugMap) {
        if (screenResponse == null) {
            return null;
        }
        DrugCheckResultsVo<DrugTherapyCheckResultVo> vo = new DrugCheckResultsVo<DrugTherapyCheckResultVo>();

        for (DPTScreenResult result : screenResponse.getDPTScreenResults()) {

            for (DPTClass dptClass : result.getDPTClasses()) {
                DrugTherapyCheckResultVo check = new DrugTherapyCheckResultVo();
                check.setAllowance(dptClass.getDuplicateAllowance());
                // hardcoded since FDB Cloud does not support custom
                check.setCustom(false);
                check.setDrugs(new ArrayList<DrugCheckVo>(result.getScreenDrugs().size()));
                for (ScreenDrug screenDrug : result.getScreenDrugs()) {
                    DrugCheckVo drug = drugMap.get(screenDrug.getTag());
                    check.getDrugs().add(drug);
                }
                check.setDuplicateClass(StringUtility.nullToEmptyString(dptClass.getDptClassDesc()));
                check.setId(Long.valueOf(dptClass.getDptClassID()));
                check.setShortText(StringUtility.nullToEmptyString(result.getScreenMessage()));
                vo.getChecks().add(check);
            }
        }
        return vo;
    }

    /**
     * Gets the gets the professional monograph capability.
     *
     * @return the gets the professional monograph capability
     */
    public final GetProfessionalMonographCapability getGetProfessionalMonographCapability() {
        return getProfessionalMonographCapability;
    }

    /**
     * Sets the gets the professional monograph capability.
     *
     * @param getProfessionalMonographCapability the new gets the professional
     *            monograph capability
     */
    public final void setGetProfessionalMonographCapability(
            GetProfessionalMonographCapability getProfessionalMonographCapability) {
        this.getProfessionalMonographCapability = getProfessionalMonographCapability;
    }

    /**
     * Gets the screen capability.
     *
     * @return the screen capability
     */
    public final ScreenCapability getScreenCapability() {
        return screenCapability;
    }

    /**
     * Sets the screen capability.
     *
     * @param screenCapability the new screen capability
     */
    public final void setScreenCapability(ScreenCapability screenCapability) {
        this.screenCapability = screenCapability;
    }

}
