package gov.va.med.pharmacy.peps.external.common.fdbcloud.capability.impl.rest;

import gov.va.med.pharmacy.peps.common.exception.DataNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.InterfaceException;
import gov.va.med.pharmacy.peps.external.common.fdbcloud.model.ResponseBase;

import java.util.Arrays;
import java.util.Map;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

/**
 * Base class for classes that invoke FDB Cloud Connector REST Services.
 * 
 * @param <REQ>
 *            The Request model object. For HTTP POST services, there should be
 *            specific model objects for the request. Use String for the
 *            services that use HTTP GET.
 * @param <RESP>
 *            The Response model object, which is a model object to which JAXB
 *            will map the JSON response.
 */
public abstract class FDBCloudRestBase<REQ, RESP extends ResponseBase> {

    /**
     * Template for common URI Query Params that all HTTP GET service calls need
     * to include.
     */
    protected static final String URI_COMMON_QUERY_PARMS = "callSystemName={callSystemName}";
    private static final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger(FDBCloudRestBase.class);
    private String baseURL;

    /**
     * Makes a call to the FDB Cloud Connector based on the specified request
     * and url variables.
     * 
     * @param request
     *            Request - for POST services, this is a custom model object,
     *            for GET services, it is String
     * @param urlVariables
     *            Map of values to use to fill in placeholders in the URI
     *            Template
     * @return RESP The Response
     * @throws DataNotFoundException
     *             If the HTTP Status code indicates Data Not Found.
     */
    public RESP call(REQ request, Map<String, ?> urlVariables) throws DataNotFoundException {
        String authScheme = "SHAREDKEY";

        String clientid = "1006";

        String secret = "VvOy/FyATuUvt5MtzIkmqFm/AtVeFvsNMjj9Bk/G/5M=";

        String authString = authScheme + " " + clientid + ":" + secret;

        RestTemplate restTemplate = new RestTemplate();
        restTemplate.getMessageConverters().add(new MappingJackson2HttpMessageConverter());
        HttpHeaders headers = createHeader(authString);
        HttpEntity<REQ> requestEntity = new HttpEntity<REQ>(request, headers);
        ResponseEntity<RESP> response = null;
        logCallParameters(request, urlVariables);

        try {
            if (urlVariables == null) {
                response = restTemplate.exchange(getUriTemplate(), getHttpMethod(), requestEntity, getResponseType());
            } else {
                response = restTemplate.exchange(getUriTemplate(), getHttpMethod(), requestEntity, getResponseType(),
                        urlVariables);
            }
            logResponse(response);
        } catch (HttpStatusCodeException e) {
            if (HttpStatus.NOT_FOUND.equals(e.getStatusCode())) {
                LOG.warn("Data Not Found.");
                throw new DataNotFoundException(e);
            }
            throw new InterfaceException(e, InterfaceException.INTERFACE_ERROR, InterfaceException.FDB_CLOUD);
        } catch (RestClientException e) {
            throw new InterfaceException(e, InterfaceException.INTERFACE_ERROR, InterfaceException.FDB_CLOUD);
        }

        RESP result = null;
        if (response != null) {
            result = response.getBody();
        }
        return result;
    }

    /**
     * Log the call response if debug is enabled.
     * 
     * @param response
     *            The response to log
     */
    private void logResponse(ResponseEntity<RESP> response) {
        if (LOG.is   ugEnabled()) {
            if (response != null) {
                LOG.debug("Response from FDBCloud: " + response);
            }
        }
    }

    /**
     * Log the call request parameters if debug is enabled.
     * 
     * @param request
     *            The request to log
     * @param urlVariables
     *            the urlVariables to log.
     */
    private void logCallParameters(REQ request, Map<String, ?> urlVariables) {
        if (LOG.is   ugEnabled()) {
            if (request != null) {
                LOG.debug("Request sent to FDBCloud: " + request);
            }
            if (urlVariables != null) {
                LOG.debug("Request variables sent to FDBCloud: " + urlVariables);
            }
        }
    }

    /**
     * Get the base of the URL for connecting to the FDB Cloud Connector
     * services.
     * 
     * @return the Base URL String
     */
    protected String getBaseURL() {
        if (baseURL == null) {
            baseURL = "https://api.fdbcloudconnector.com/CC/api/v1_3/";
        }
        return baseURL;
    }

    /**
     * Set the base of the URL for connecting to the FDB Cloud Connector
     * services.
     * 
     * @param baseURL
     *            The new Base URL String value
     */
    public void setBaseURL(String baseURL) {
        this.baseURL = baseURL;
    }

    /**
     * Creates the HTTP Header for the request, setting the FDB Cloud
     * authorization and other global parameters.
     * 
     * @param authString
     *            The authorization string to use in the header
     * @return HttpHeaders containing all the header information needed for a
     *         call to FDB Cloud Connector.
     */
    protected HttpHeaders createHeader(String authString) {
        HttpHeaders headers = new HttpHeaders();
        headers.setAccept(Arrays.asList(org.springframework.http.MediaType.APPLICATION_JSON));
        headers.add("Authorization", authString);
        headers.add("fdb-UseCamelCaseforJson", "true");
        return headers;
    }

    /**
     * All FDB Cloud services use GET except for Screen. Since the majority use
     * Get, we provide it here and allow the other services to override.
     * 
     * @return HttpMethod to use for the service
     */
    protected HttpMethod getHttpMethod() {
        return HttpMethod.GET;
    }

    /**
     * Since Java Generics cannot provide an instance of Class<RESP>, we need
     * subclasses to provide it.
     * 
     * @return Class<RESP>
     */
    protected abstract Class<RESP> getResponseType();

    /**
     * Gets the complete uri template for the service. If placeholders are
     * included in the template, then matching urlVariables should be provided
     * in the invocation of the call method.
     *
     * @return the uri template
     */
    protected abstract String getUriTemplate();

}
