package gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.test;

import gov.va.med.pharmacy.peps.common.vo.DrugCheckMessageVo;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckVo;
import gov.va.med.pharmacy.peps.common.vo.DrugDrugCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.DrugTherapyCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckVo;
import gov.va.med.pharmacy.peps.common.vo.ProfessionalMonographVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.PerformDrugChecksCapability;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import junit.framework.TestCase;

import org.apache.commons.lang.ObjectUtils;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.context.support.GenericXmlApplicationContext;

/**
 * Performs a comparison of the results of two different
 * DrugTherapyCheckCapability implementations. In this case, it compares FDBDIF
 * API versus FDB Cloud Connector, but the test can be easily used for any two
 * implementations, which are designated in the setUp method.
 */
public class PerformDrugChecksCapabilityComparisonTest extends TestCase {
    private static final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
            .getLogger(PerformDrugChecksCapabilityComparisonTest.class);
    private static final String LINE_OF_EQUALS = System.lineSeparator()
            + "======================================================================";
    private PerformDrugChecksCapability capabilityFDBDIF;
    private PerformDrugChecksCapability capabilityFDBCloud;

    /**
     * Load the capability implementations from Spring contexts.
     * 
     * @throws Exception
     *             If error in opening the contexts.
     * @see junit.framework.TestCase#setUp()
     */
    @SuppressWarnings("resource")
    protected void setUp() throws Exception {
        LOG.debug("--------------------------" + getName() + "--------------------------");

        GenericXmlApplicationContext contextFDBDIF = new GenericXmlApplicationContext(
                "spring/mocha/test/InterfaceContext-NoImport.xml", "spring/InterfaceContextTemplate.xml");
        this.capabilityFDBDIF = (PerformDrugChecksCapability) contextFDBDIF.getBean("performDrugChecksCapability");

        GenericXmlApplicationContext contextFDBCloud = new GenericXmlApplicationContext(
                "spring/mocha/test/InterfaceContext-NoImport.xml", "spring/InterfaceContextTemplate-FDBCloud.xml");
        this.capabilityFDBCloud = (PerformDrugChecksCapability) contextFDBCloud.getBean("performDrugChecksCapability");
    }

    /**
     * Duplicate Therapy Test - Tests that the same drug twice will cause a
     * duplicate therapy warning.
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugTherapyCheckSameDrug() throws IllegalArgumentException, IllegalAccessException {
        OrderCheckVo checkVo = createDuplicateTherapyOrderCheckVo("12000", "12000");
        executeTest(checkVo, 0, 0);
    }

    /**
     * Duplicate Therapy Test - Tests that results are returned successfully
     * with warnings for three of the four drugs, and the fourth is an invalid
     * one that gets reported as a drug not checked.
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugTherapyCheckFourDrugsOneUnknown() throws IllegalArgumentException,
            IllegalAccessException {
        OrderCheckVo checkVo = createDuplicateTherapyOrderCheckVo("12000", "6558", "6559", "99999");
        executeTest(checkVo, 1, 0);
    }

    /**
     * Duplicate Therapy Test - Tests that results are returned successfully
     * with No warnings.
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugTherapyCheckNoWarnings() throws IllegalArgumentException, IllegalAccessException {
        OrderCheckVo checkVo = createDuplicateTherapyOrderCheckVo("12000", "10", "6693");
        executeTest(checkVo, 0, 0);
    }

    /**
     * Drug Drug Interaction Test - Tests two drugs that have an interaction
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugDrugCheckWithInteraction() throws IllegalArgumentException, IllegalAccessException {
        OrderCheckVo checkVo = createDDIOrderCheckVo("10", "2025");
        executeTest(checkVo, 0, 0);
    }

    /**
     * Drug Drug Interaction Test - Tests two drugs that have an interaction
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugDrugCheckWithNoInteraction() throws IllegalArgumentException, IllegalAccessException {
        OrderCheckVo checkVo = createDDIOrderCheckVo("10", "6558");
        executeTest(checkVo, 0, 0);
    }

    /**
     * Drug Drug Interaction Test - Tests two drugs that have an interaction
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugDrugCheckMultipleDrugsWithInteractions() throws IllegalArgumentException,
            IllegalAccessException {
        OrderCheckVo checkVo = createDDIOrderCheckVo("10", "67810", "21438", "9");
        executeTest(checkVo, 0, 0);
    }

    /**
     * Verify execute method returns some results
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    public void executeTest(OrderCheckVo checkVo, int expectedDrugsNotCheckedCount, int expectedMessagesCount)
            throws IllegalArgumentException, IllegalAccessException {

        LOG.debug("Comparing results from " + capabilityFDBDIF.getClass().getName() + " and "
                + capabilityFDBCloud.getClass().getName());

        OrderCheckResultsVo resultsFDBDIF = capabilityFDBDIF.performDrugChecks(checkVo);
        LOG.trace(resultsFDBDIF);

        assertExpectedDrugsNotCheckedCount(expectedDrugsNotCheckedCount, resultsFDBDIF.getDrugsNotChecked());

        if (checkVo.isDrugTherapyCheck()) {
            assertNotNull(resultsFDBDIF.getDrugTherapyCheckResults());
            assertNotNull("DrugTherapyCheckResults().getChecks() is null when expected value", resultsFDBDIF
                    .getDrugTherapyCheckResults().getChecks());
            assertTrue("DrugTherapyCheckResults().getChecks() is empty", resultsFDBDIF.getDrugTherapyCheckResults()
                    .getChecks().size() > 0);
        }

        OrderCheckResultsVo resultsFDBCloud = capabilityFDBCloud.performDrugChecks(checkVo);
        LOG.trace(resultsFDBCloud);

        assertExpectedDrugsNotCheckedCount(expectedDrugsNotCheckedCount, resultsFDBCloud.getDrugsNotChecked());

        compareResults(checkVo, resultsFDBDIF, resultsFDBCloud);
    }

    /**
     * @param expectedDrugsNotCheckedCount
     * @param drugsNotChecked
     */
    private void assertExpectedDrugsNotCheckedCount(int expectedDrugsNotCheckedCount,
            Collection<DrugCheckVo> drugsNotChecked) {
        if (expectedDrugsNotCheckedCount > 0) {
            assertNotNull("drugsNotChecked is null when expected value." + drugsNotChecked);
            assertTrue("Expected drugsNotChecked to contain " + expectedDrugsNotCheckedCount + " record(s).",
                    drugsNotChecked.size() == 1);
        }
    }

    /**
     * Creates an OrderCheckVo for an Order Check, using the list of gcnseqnos
     * specified. Caller needs to set what type of order check and any
     * check-specific parameters.
     * 
     * @param gcnSeqNos
     *            List of gcnseqnos of the drugs to include in the order check.
     * @return OrderCheckVo - note that the type of order check is not set,
     *         caller needs to set.
     */
    private OrderCheckVo createOrderCheckVo(String... gcnSeqNos) {
        Collection<DrugCheckVo> drugsToScreen = new ArrayList<DrugCheckVo>();

        for (String gcnSeqNo : gcnSeqNos) {
            DrugCheckVo drugCheckVo = new DrugCheckVo();
            drugCheckVo.setGcnSeqNo(gcnSeqNo);
            drugCheckVo.setProspective(false);
            drugsToScreen.add(drugCheckVo);
        }

        OrderCheckVo checkVo = new OrderCheckVo();
        checkVo.setDrugsToScreen(drugsToScreen);
        checkVo.setProspectiveOnly(false);
        return checkVo;
    }

    /**
     * Creates an OrderCheckVo for a Duplicate Therapy Order Check, using the
     * list of gcnseqnos specified.
     */
    private OrderCheckVo createDuplicateTherapyOrderCheckVo(String... gcnSeqNos) {
        OrderCheckVo checkVo = createOrderCheckVo(gcnSeqNos);
        checkVo.setDuplicateAllowance(false);
        checkVo.setDrugTherapyCheck(true);

        return checkVo;
    }

    /**
     * Creates an OrderCheckVo for a Duplicate Therapy Order Check, using the
     * list of gcnseqnos specified.
     */
    private OrderCheckVo createDDIOrderCheckVo(String... gcnSeqNos) {
        OrderCheckVo checkVo = createOrderCheckVo(gcnSeqNos);
        checkVo.setDrugDrugCheck(true);

        return checkVo;
    }

    /**
     * Compares two OrderCheckResultsVo.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareResults(OrderCheckVo request, OrderCheckResultsVo resultsFDBDIF,
            OrderCheckResultsVo resultsFDBCloud) {

        compareDrugsNotChecked(resultsFDBDIF.getDrugsNotChecked(), resultsFDBCloud.getDrugsNotChecked());

        if (request.isDrugTherapyCheck()) {
            compareMessages(resultsFDBDIF.getDrugTherapyCheckResults().getMessages(), resultsFDBCloud
                    .getDrugTherapyCheckResults().getMessages());

            compareDTCheckResults(resultsFDBDIF.getDrugTherapyCheckResults().getChecks(), resultsFDBCloud
                    .getDrugTherapyCheckResults().getChecks());
        }

        if (request.isDrugDrugCheck()) {
            compareMessages(resultsFDBDIF.getDrugDrugCheckResults().getMessages(), resultsFDBCloud
                    .getDrugDrugCheckResults().getMessages());

            compareDrugDrugCheckResults(resultsFDBDIF.getDrugDrugCheckResults().getChecks(), resultsFDBCloud
                    .getDrugDrugCheckResults().getChecks());
        }

    }

    /**
     * Compares two drugsNotChecked lists. Fails test if the lists are not of
     * equal size with gcnseqnos from one list matching all those in the other
     * list. Logs field differences.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareMessages(Collection<DrugCheckMessageVo> messagesFDBDIF,
            Collection<DrugCheckMessageVo> messagesFDBCloud) {
        if (messagesFDBDIF == null && messagesFDBCloud == null) {
            return;
        } else if (messagesFDBDIF == null && messagesFDBCloud != null) {
            LOG.error("FDBDIF does not have messages, but FDBCloud has messages.");
            return;
        } else if (messagesFDBDIF != null && messagesFDBCloud == null) {
            LOG.error("FDBDIF has messages, but FDBCloud does not have messages.");
            return;
        }

        Assert.assertEquals("messages Lists do not match.", messagesFDBDIF.size(), messagesFDBCloud.size());
        for (DrugCheckMessageVo messageFDBCloud : messagesFDBCloud) {
            boolean foundMatch = false;
            for (DrugCheckMessageVo messageFDBDIF : messagesFDBDIF) {
                if (messageFDBDIF.getDrug().getGcnSeqNo().equals(messageFDBCloud.getDrug().getGcnSeqNo())) {
                    foundMatch = true;
                    LOG.debug("Comparing DrugsNotChecked with gcnseqno: " + messageFDBDIF.getDrug().getGcnSeqNo());

                    compareDrugCheck(messageFDBDIF.getDrug(), messageFDBCloud.getDrug());

                    if (messageFDBDIF.getDrugName() != messageFDBCloud.getDrugName()) {
                        logDifference("drugName", messageFDBDIF.getDrugName(), messageFDBCloud.getDrugName());
                    }
                    if (messageFDBDIF.getSeverity() != messageFDBCloud.getSeverity()) {
                        logDifference("severity", messageFDBDIF.getSeverity(), messageFDBCloud.getSeverity());
                    }
                    if (messageFDBDIF.getText() != messageFDBCloud.getText()) {
                        logDifference("text", messageFDBDIF.getText(), messageFDBCloud.getText());
                    }
                    if (messageFDBDIF.getType() != messageFDBCloud.getType()) {
                        logDifference("type", messageFDBDIF.getType(), messageFDBCloud.getType());
                    }
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a matching message in FDBDIF for FDBCloud record: "
                        + messageFDBCloud.getDrug().getGcnSeqNo());
            }
        }
    }

    /**
     * Compares two drugsNotChecked lists. Fails test if the lists are not of
     * equal size with gcnseqnos from one list matching all those in the other
     * list. Logs field differences.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareDrugsNotChecked(Collection<DrugCheckVo> drugsNotCheckedFDBDIF,
            Collection<DrugCheckVo> drugsNotCheckedFDBCloud) {
        if (drugsNotCheckedFDBDIF == null && drugsNotCheckedFDBCloud == null) {
            return;
        } else if (drugsNotCheckedFDBDIF == null && drugsNotCheckedFDBCloud != null) {
            LOG.error("FDBDIF does not have drugsNotChecked, but FDBCloud has drugsNotChecked.");
            return;
        } else if (drugsNotCheckedFDBDIF != null && drugsNotCheckedFDBCloud == null) {
            LOG.error("FDBDIF has drugsNotChecked, but FDBCloud does not have drugsNotChecked.");
            return;
        }
        Assert.assertEquals("drugsNotChecked Lists do not match.", drugsNotCheckedFDBDIF.size(),
                drugsNotCheckedFDBCloud.size());
        for (DrugCheckVo drugCheckVoFDBCloud : drugsNotCheckedFDBCloud) {
            boolean foundMatch = false;
            for (DrugCheckVo drugCheckVoFDBDIF : drugsNotCheckedFDBDIF) {
                if (drugCheckVoFDBDIF.getGcnSeqNo().equals(drugCheckVoFDBCloud.getGcnSeqNo())) {
                    foundMatch = true;
                    compareDrugCheck(drugCheckVoFDBCloud, drugCheckVoFDBDIF);
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a matching drugNotChecked in FDBDIF for FDBCloud record: "
                        + drugCheckVoFDBCloud.getGcnSeqNo());
            }
        }
    }

    /**
     * Compares two drugChecks lists. Fails test if the lists are not of equal
     * size with gcnseqnos from one list matching all those in the other list.
     * Logs field differences.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareDrugChecks(Collection<DrugCheckVo> drugChecksFDBDIF, Collection<DrugCheckVo> drugChecksFDBCloud) {
        if (drugChecksFDBDIF == null && drugChecksFDBCloud == null) {
            return;
        } else if (drugChecksFDBDIF == null && drugChecksFDBCloud != null) {
            LOG.error("FDBDIF does not have drugChecks, but FDBCloud has drugChecks.");
            return;
        } else if (drugChecksFDBDIF != null && drugChecksFDBCloud == null) {
            LOG.error("FDBDIF has drugChecks, but FDBCloud does not have drugChecks.");
            return;
        }
        Assert.assertEquals("drugChecks Lists do not match.", drugChecksFDBDIF.size(), drugChecksFDBCloud.size());
        for (DrugCheckVo drugCheckVoFDBCloud : drugChecksFDBCloud) {
            boolean foundMatch = false;
            for (DrugCheckVo drugCheckVoFDBDIF : drugChecksFDBDIF) {
                if (drugCheckVoFDBDIF.getGcnSeqNo().equals(drugCheckVoFDBCloud.getGcnSeqNo())) {
                    foundMatch = true;
                    compareDrugCheck(drugCheckVoFDBCloud, drugCheckVoFDBDIF);
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a matching drugCheck in FDBDIF for FDBCloud record: "
                        + drugCheckVoFDBCloud.getGcnSeqNo());
            }
        }
    }

    /**
     * @param drugCheckVoFDBCloud
     * @param drugCheckVoFDBDIF
     */
    private void compareDrugCheck(DrugCheckVo drugCheckVoFDBCloud, DrugCheckVo drugCheckVoFDBDIF) {
        LOG.debug("Comparing DrugsNotChecked with gcnseqno: " + drugCheckVoFDBDIF.getGcnSeqNo());

        if (drugCheckVoFDBDIF.isDoseCheck() != drugCheckVoFDBCloud.isDoseCheck()) {
            logDifference("doseCheck", drugCheckVoFDBDIF.isDoseCheck(), drugCheckVoFDBCloud.isDoseCheck());
        }
        if (drugCheckVoFDBDIF.isProspective() != drugCheckVoFDBCloud.isProspective()) {
            logDifference("doseCheck", drugCheckVoFDBDIF.isProspective(), drugCheckVoFDBCloud.isProspective());
        }
        if (drugCheckVoFDBDIF.getDoseRate() != drugCheckVoFDBCloud.getDoseRate()) {
            logDifference("doseRate", drugCheckVoFDBDIF.getDoseRate(), drugCheckVoFDBCloud.getDoseRate());
        }
        if (drugCheckVoFDBDIF.getDoseRoute() != drugCheckVoFDBCloud.getDoseRoute()) {
            logDifference("doseRate", drugCheckVoFDBDIF.getDoseRoute(), drugCheckVoFDBCloud.getDoseRoute());
        }
        if (drugCheckVoFDBDIF.getDoseType() != drugCheckVoFDBCloud.getDoseType()) {
            logDifference("doseType", drugCheckVoFDBDIF.getDoseType(), drugCheckVoFDBCloud.getDoseType());
        }
        if (drugCheckVoFDBDIF.getDoseUnit() != drugCheckVoFDBCloud.getDoseUnit()) {
            logDifference("doseUnit", drugCheckVoFDBDIF.getDoseUnit(), drugCheckVoFDBCloud.getDoseUnit());
        }
        if (drugCheckVoFDBDIF.getDrugName() != drugCheckVoFDBCloud.getDrugName()) {
            logDifference("drugName", drugCheckVoFDBDIF.getDrugName(), drugCheckVoFDBCloud.getDrugName());
        }
        if (drugCheckVoFDBDIF.getDuration() != drugCheckVoFDBCloud.getDuration()) {
            logDifference("duration", drugCheckVoFDBDIF.getDuration(), drugCheckVoFDBCloud.getDuration());
        }
        if (drugCheckVoFDBDIF.getDurationRate() != drugCheckVoFDBCloud.getDurationRate()) {
            logDifference("durationRate", drugCheckVoFDBDIF.getDurationRate(), drugCheckVoFDBCloud.getDurationRate());
        }
        if (drugCheckVoFDBDIF.getFrequency() != drugCheckVoFDBCloud.getFrequency()) {
            logDifference("frequency", drugCheckVoFDBDIF.getFrequency(), drugCheckVoFDBCloud.getFrequency());
        }
        if (drugCheckVoFDBDIF.getIen() != drugCheckVoFDBCloud.getIen()) {
            logDifference("IEN", drugCheckVoFDBDIF.getIen(), drugCheckVoFDBCloud.getIen());
        }
        if (drugCheckVoFDBDIF.getOrderNumber() != drugCheckVoFDBCloud.getOrderNumber()) {
            logDifference("orderNumber", drugCheckVoFDBDIF.getOrderNumber(), drugCheckVoFDBCloud.getOrderNumber());
        }
        if (drugCheckVoFDBDIF.getSingleDoseAmount() != drugCheckVoFDBCloud.getSingleDoseAmount()) {
            logDifference("singleDoseAmount", drugCheckVoFDBDIF.getSingleDoseAmount(),
                    drugCheckVoFDBCloud.getSingleDoseAmount());
        }
        if (drugCheckVoFDBDIF.getVuid() != drugCheckVoFDBCloud.getVuid()) {
            logDifference("VUID", drugCheckVoFDBDIF.getVuid(), drugCheckVoFDBCloud.getVuid());
        }
    }

    /**
     * @param drugChecksFDBDIF
     * @param drugChecksFDBCloud
     */
    private void compareDTCheckResults(Collection<DrugTherapyCheckResultVo> drugChecksFDBDIF,
            Collection<DrugTherapyCheckResultVo> drugChecksFDBCloud) {
        if (drugChecksFDBDIF == null && drugChecksFDBCloud == null) {
            return;
        } else if (drugChecksFDBDIF == null && drugChecksFDBCloud != null) {
            LOG.error("FDBDIF does not have DrugTherapyCheckResultVOs, but FDBCloud has DrugTherapyCheckResultVos.");
            return;
        } else if (drugChecksFDBDIF != null && drugChecksFDBCloud == null) {
            LOG.error("FDBDIF has DrugTherapyCheckResultVos, but FDBCloud does not have DrugTherapyCheckResultVos.");
            return;
        }

        for (DrugTherapyCheckResultVo dtCheckResultVoFDBCloud : drugChecksFDBCloud) {
            boolean foundMatch = false;
            for (DrugTherapyCheckResultVo dtCheckResultVoFDBDIF : drugChecksFDBDIF) {
                if (dtCheckResultVoFDBDIF.getId() == dtCheckResultVoFDBCloud.getId()) {
                    foundMatch = true;

                    LOG.debug(LINE_OF_EQUALS + System.lineSeparator() + "Comparing DrugTherapyCheckResultVo with id: "
                            + dtCheckResultVoFDBCloud.getId() + LINE_OF_EQUALS);

                    if (dtCheckResultVoFDBDIF.isCustom() != dtCheckResultVoFDBCloud.isCustom()) {
                        logDifference("isCustom", dtCheckResultVoFDBDIF.isCustom(), dtCheckResultVoFDBCloud.isCustom());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getDuplicateClass(),
                            dtCheckResultVoFDBCloud.getDuplicateClass())) {
                        logDifference("duplicateClass", dtCheckResultVoFDBDIF.getDuplicateClass(),
                                dtCheckResultVoFDBCloud.getDuplicateClass());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getShortText(),
                            dtCheckResultVoFDBCloud.getShortText())) {
                        logDifference("shortText", dtCheckResultVoFDBDIF.getShortText(),
                                dtCheckResultVoFDBCloud.getShortText());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getAllowance(),
                            dtCheckResultVoFDBCloud.getAllowance())) {
                        logDifference("allowance", dtCheckResultVoFDBDIF.getAllowance(),
                                dtCheckResultVoFDBCloud.getAllowance());
                    }

                    compareDrugChecks(dtCheckResultVoFDBDIF.getDrugs(), dtCheckResultVoFDBCloud.getDrugs());
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a match in FDBDIF for FDBCloud record: " + dtCheckResultVoFDBCloud);
            }
        }
    }

    /**
     * @param drugChecksFDBDIF
     * @param drugChecksFDBCloud
     */
    private void compareDrugDrugCheckResults(Collection<DrugDrugCheckResultVo> drugChecksFDBDIF,
            Collection<DrugDrugCheckResultVo> drugChecksFDBCloud) {
        if (drugChecksFDBDIF == null && drugChecksFDBCloud == null) {
            return;
        } else if (drugChecksFDBDIF == null && drugChecksFDBCloud != null) {
            LOG.error("FDBDIF does not have DrugDrugCheckResultVos, but FDBCloud has DrugDrugCheckResultVos.");
            return;
        } else if (drugChecksFDBDIF != null && drugChecksFDBCloud == null) {
            LOG.error("FDBDIF has DrugDrugCheckResultVos, but FDBCloud does not have DrugDrugCheckResultVos.");
            return;
        }

        for (DrugDrugCheckResultVo ddiCheckResultVoFDBCloud : drugChecksFDBCloud) {
            boolean foundMatch = false;
            for (DrugDrugCheckResultVo ddiCheckResultVoFDBDIF : drugChecksFDBDIF) {
                if (isSameInteraction(ddiCheckResultVoFDBCloud, ddiCheckResultVoFDBDIF)) {
                    foundMatch = true;
                    compareDrugDrugCheckResultVo(ddiCheckResultVoFDBCloud, ddiCheckResultVoFDBDIF);
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a match in FDBDIF for FDBCloud record: " + ddiCheckResultVoFDBCloud);
            }
        }
    }

    private boolean isSameInteraction(DrugDrugCheckResultVo ddiCheckResultVoFDBCloud,
            DrugDrugCheckResultVo ddiCheckResultVoFDBDIF) {
        boolean isMatch = false;
        if (ddiCheckResultVoFDBDIF.getId() == ddiCheckResultVoFDBCloud.getId()) {

            Set<String> fdbDifDrugGcnSeqNoList = new HashSet<String>();
            for (DrugCheckVo drugCheckVoFDBDIF : ddiCheckResultVoFDBDIF.getDrugs()) {
                fdbDifDrugGcnSeqNoList.add(drugCheckVoFDBDIF.getGcnSeqNo());
            }

            for (DrugCheckVo drugCheckVoFDBCloud : ddiCheckResultVoFDBCloud.getDrugs()) {
                if (fdbDifDrugGcnSeqNoList.contains(drugCheckVoFDBCloud.getGcnSeqNo())) {
                    fdbDifDrugGcnSeqNoList.remove(drugCheckVoFDBCloud.getGcnSeqNo());
                } else {
                    return false;
                }
            }
            // if we burned off the whole list, then we have a match
            isMatch = fdbDifDrugGcnSeqNoList.isEmpty();
        }
        return isMatch;
    }

    /**
     * @param ddiCheckResultVoFDBCloud
     * @param ddiCheckResultVoFDBDIF
     */
    private void compareDrugDrugCheckResultVo(DrugDrugCheckResultVo ddiCheckResultVoFDBCloud,
            DrugDrugCheckResultVo ddiCheckResultVoFDBDIF) {

        String gcnSeqNoLogLine = " GCNSeqNos: ";
        for (DrugCheckVo drugCheckVo : ddiCheckResultVoFDBCloud.getDrugs()) {
            gcnSeqNoLogLine += drugCheckVo.getGcnSeqNo() + " ";
        }
        LOG.debug(LINE_OF_EQUALS + System.lineSeparator() + "Comparing DrugDrugCheckResultVo with id: "
                + ddiCheckResultVoFDBCloud.getId() + gcnSeqNoLogLine + LINE_OF_EQUALS);

        if (ddiCheckResultVoFDBDIF.isCustom() != ddiCheckResultVoFDBCloud.isCustom()) {
            logDifference("isCustom", ddiCheckResultVoFDBDIF.isCustom(), ddiCheckResultVoFDBCloud.isCustom());
        }

        if (!ObjectUtils.equals(ddiCheckResultVoFDBDIF.getConsumerMonograph(),
                ddiCheckResultVoFDBCloud.getConsumerMonograph())) {
            logDifference("consumerMonograph", ddiCheckResultVoFDBDIF.getConsumerMonograph(),
                    ddiCheckResultVoFDBCloud.getConsumerMonograph());
        }

        if (!ObjectUtils.equals(ddiCheckResultVoFDBDIF.getInteractionDescription(),
                ddiCheckResultVoFDBCloud.getInteractionDescription())) {
            logDifference("interactionDescription", ddiCheckResultVoFDBDIF.getInteractionDescription(),
                    ddiCheckResultVoFDBCloud.getInteractionDescription());
        }

        compareProfessionalMonograph(ddiCheckResultVoFDBDIF.getProfessionalMonograph(),
                ddiCheckResultVoFDBCloud.getProfessionalMonograph());

        if (!ObjectUtils.equals(ddiCheckResultVoFDBDIF.getSeverity(), ddiCheckResultVoFDBCloud.getSeverity())) {
            logDifference("severity", ddiCheckResultVoFDBDIF.getSeverity(), ddiCheckResultVoFDBCloud.getSeverity());
        }

        if (!ObjectUtils.equals(ddiCheckResultVoFDBDIF.getShortText(), ddiCheckResultVoFDBCloud.getShortText())) {
            logDifference("shortText", ddiCheckResultVoFDBDIF.getShortText(), ddiCheckResultVoFDBCloud.getShortText());
        }

        compareDrugChecks(ddiCheckResultVoFDBDIF.getDrugs(), ddiCheckResultVoFDBCloud.getDrugs());
    }

    private void compareProfessionalMonograph(ProfessionalMonographVo professionalMonographFDBDIF,
            ProfessionalMonographVo professionalMonographFDBCloud) {

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getClinicalEffects(),
                professionalMonographFDBCloud.getClinicalEffects())) {
            logDifference("clinicalEffects", professionalMonographFDBDIF.getClinicalEffects(),
                    professionalMonographFDBCloud.getClinicalEffects());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getDisclaimer(),
                professionalMonographFDBCloud.getDisclaimer())) {
            logDifference("disclaimer", professionalMonographFDBDIF.getDisclaimer(),
                    professionalMonographFDBCloud.getDisclaimer());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getDiscussion(),
                professionalMonographFDBCloud.getDiscussion())) {
            logDifference("discussion", professionalMonographFDBDIF.getDiscussion(),
                    professionalMonographFDBCloud.getDiscussion());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getMechanismOfAction(),
                professionalMonographFDBCloud.getMechanismOfAction())) {
            logDifference("mechanismOfAction", professionalMonographFDBDIF.getMechanismOfAction(),
                    professionalMonographFDBCloud.getMechanismOfAction());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getMonographTitle(),
                professionalMonographFDBCloud.getMonographTitle())) {
            logDifference("monographTitle", professionalMonographFDBDIF.getMonographTitle(),
                    professionalMonographFDBCloud.getMonographTitle());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getPatientManagement(),
                professionalMonographFDBCloud.getPatientManagement())) {
            logDifference("patientManagement", professionalMonographFDBDIF.getPatientManagement(),
                    professionalMonographFDBCloud.getPatientManagement());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getPredisposingFactors(),
                professionalMonographFDBCloud.getPredisposingFactors())) {
            logDifference("predisposingFactors", professionalMonographFDBDIF.getPredisposingFactors(),
                    professionalMonographFDBCloud.getPredisposingFactors());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getReferences(),
                professionalMonographFDBCloud.getReferences())) {
            logDifference("references", professionalMonographFDBDIF.getReferences(),
                    professionalMonographFDBCloud.getReferences());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.getSeverityLevel(),
                professionalMonographFDBCloud.getSeverityLevel())) {
            logDifference("severityLevel", professionalMonographFDBDIF.getSeverityLevel(),
                    professionalMonographFDBCloud.getSeverityLevel());
        }

        if (!ObjectUtils.equals(professionalMonographFDBDIF.isFdbMonographSourceType(),
                professionalMonographFDBCloud.isFdbMonographSourceType())) {
            logDifference("isFdbMonographSourceType", professionalMonographFDBDIF.isFdbMonographSourceType(),
                    professionalMonographFDBCloud.isFdbMonographSourceType());
        }
    }

    private void logDifference(String fieldName, Object fdbDIFValue, Object fdbCloudValue) {
        LOG.warn("DIFF in " + fieldName + ":");
        LOG.warn("FDBDIF   : " + fdbDIFValue);
        LOG.warn("FDBCloud : " + fdbCloudValue);
    }
}