/**
 * Copyright 2006, Southwest Research Institute
 */

package gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.test;

import gov.va.med.pharmacy.peps.common.vo.DrugCheckMessageVo;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckVo;
import gov.va.med.pharmacy.peps.common.vo.DrugTherapyCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.PerformDrugChecksCapability;

import java.util.ArrayList;
import java.util.Collection;

import junit.framework.TestCase;

import org.apache.commons.lang.ObjectUtils;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.context.support.GenericXmlApplicationContext;

/**
 * Validate drug-therapy check returns some results
 */
public class PerformDrugTherapyCheckCommandTest extends TestCase {
    private static final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
            .getLogger(PerformDrugTherapyCheckCommandTest.class);
    private PerformDrugChecksCapability capabilityFDBDIF;
    private PerformDrugChecksCapability capabilityFDBCloud;

    /**
     * Instantiate and add drugs to drugsToScreen collection
     * 
     * @throws Exception
     *             If error in setting up collection
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @SuppressWarnings("resource")
    protected void setUp() throws Exception {
        LOG.trace("--------------------------" + getName() + "--------------------------");

        GenericXmlApplicationContext contextFDBDIF = new GenericXmlApplicationContext(
                "spring/mocha/test/InterfaceContext-NoImport.xml", "spring/InterfaceContextTemplate.xml");
        this.capabilityFDBDIF = (PerformDrugChecksCapability) contextFDBDIF.getBean("performDrugChecksCapability");

        GenericXmlApplicationContext contextFDBCloud = new GenericXmlApplicationContext(
                "spring/mocha/test/InterfaceContext-NoImport.xml", "spring/InterfaceContextTemplate-FDBCloud.xml");
        this.capabilityFDBCloud = (PerformDrugChecksCapability) contextFDBCloud.getBean("performDrugChecksCapability");
    }

    /**
     * 
     */
    private OrderCheckVo createDuplicateTherapyOrderCheckVo(String... gcnSeqNos) {
        Collection<DrugCheckVo> drugsToScreen = new ArrayList<DrugCheckVo>();

        for (String gcnSeqNo : gcnSeqNos) {
            DrugCheckVo drugCheckVo = new DrugCheckVo();
            drugCheckVo.setGcnSeqNo(gcnSeqNo);
            drugCheckVo.setProspective(false);
            drugsToScreen.add(drugCheckVo);
        }

        OrderCheckVo checkVo = new OrderCheckVo();
        checkVo.setDrugsToScreen(drugsToScreen);
        checkVo.setDuplicateAllowance(false);
        checkVo.setProspectiveOnly(false);
        checkVo.setDrugTherapyCheck(true);

        return checkVo;
    }

    /**
     * Verify execute method returns some results
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugTherapyCheckSameDrug() throws IllegalArgumentException, IllegalAccessException {
        OrderCheckVo checkVo = createDuplicateTherapyOrderCheckVo("12000", "12000");

        OrderCheckResultsVo resultsFDBDIF = capabilityFDBDIF.performDrugChecks(checkVo);

        assertTrue("Duplicate therapy check returned results", resultsFDBDIF.getDrugTherapyCheckResults().getChecks()
                .size() > 0);
        LOG.debug(resultsFDBDIF);

        OrderCheckResultsVo resultsFDBCloud = capabilityFDBCloud.performDrugChecks(checkVo);

        assertTrue("Duplicate therapy check returned results", resultsFDBCloud.getDrugTherapyCheckResults().getChecks()
                .size() > 0);
        LOG.debug(resultsFDBCloud);

        compareResults(resultsFDBDIF, resultsFDBCloud);
    }

    /**
     * Verify execute method returns some results
     * 
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     */
    @Test
    public void testCompareDrugTherapyCheckFourDrugsOneUnknown() throws IllegalArgumentException,
            IllegalAccessException {
        OrderCheckVo checkVo = createDuplicateTherapyOrderCheckVo("12000", "6558", "6559", "99999");

        OrderCheckResultsVo resultsFDBDIF = capabilityFDBDIF.performDrugChecks(checkVo);

        // assertTrue(resultsFDBDIF.getDrugsNotChecked()
        assertNotNull(resultsFDBDIF.getDrugTherapyCheckResults());
        assertNotNull(resultsFDBDIF.getDrugTherapyCheckResults().getChecks());
        assertTrue("Duplicate therapy check returned results", resultsFDBDIF.getDrugTherapyCheckResults().getChecks()
                .size() > 0);
        LOG.debug(resultsFDBDIF);

        OrderCheckResultsVo resultsFDBCloud = capabilityFDBCloud.performDrugChecks(checkVo);

        assertTrue("Duplicate therapy check returned results", resultsFDBCloud.getDrugTherapyCheckResults().getChecks()
                .size() > 0);
        LOG.debug(resultsFDBCloud);

        compareResults(resultsFDBDIF, resultsFDBCloud);
    }

    /**
     * Compares two OrderCheckResultsVo.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareResults(OrderCheckResultsVo resultsFDBDIF, OrderCheckResultsVo resultsFDBCloud) {
        Collection<DrugTherapyCheckResultVo> drugChecksFDBDIF = resultsFDBDIF.getDrugTherapyCheckResults().getChecks();
        Collection<DrugTherapyCheckResultVo> drugChecksFDBCloud = resultsFDBCloud.getDrugTherapyCheckResults()
                .getChecks();

        compareMessages(resultsFDBDIF.getDrugTherapyCheckResults().getMessages(), resultsFDBCloud
                .getDrugTherapyCheckResults().getMessages());

        compareDrugsNotChecked(resultsFDBDIF.getDrugsNotChecked(), resultsFDBCloud.getDrugsNotChecked());

        compareDTCheckResults(drugChecksFDBDIF, drugChecksFDBCloud);
    }

    /**
     * Compares two drugsNotChecked lists. Fails test if the lists are not of
     * equal size with gcnseqnos from one list matching all those in the other
     * list. Logs field differences.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareMessages(Collection<DrugCheckMessageVo> messagesFDBDIF,
            Collection<DrugCheckMessageVo> messagesFDBCloud) {
        if (messagesFDBDIF == null && messagesFDBCloud == null) {
            return;
        } else if (messagesFDBDIF == null && messagesFDBCloud != null) {
            LOG.error("FDBDIF does not have messages, but FDBCloud has messages.");
            return;
        } else if (messagesFDBDIF != null && messagesFDBCloud == null) {
            LOG.error("FDBDIF has messages, but FDBCloud does not have messages.");
            return;
        }

        Assert.assertEquals("messages Lists do not match.", messagesFDBDIF.size(), messagesFDBCloud.size());
        for (DrugCheckMessageVo messageFDBCloud : messagesFDBCloud) {
            for (DrugCheckMessageVo messageFDBDIF : messagesFDBDIF) {
                boolean foundMatch = false;
                if (messageFDBDIF.getDrug().getGcnSeqNo().equals(messageFDBCloud.getDrug().getGcnSeqNo())) {
                    foundMatch = true;
                    LOG.trace("Comparing DrugsNotChecked with gcnseqno: " + messageFDBDIF.getDrug().getGcnSeqNo());

                    compareDrugCheck(messageFDBDIF.getDrug(), messageFDBCloud.getDrug());

                    if (messageFDBDIF.getDrugName() != messageFDBCloud.getDrugName()) {
                        logDifference("drugName", messageFDBDIF.getDrugName(), messageFDBCloud.getDrugName());
                    }
                    if (messageFDBDIF.getSeverity() != messageFDBCloud.getSeverity()) {
                        logDifference("severity", messageFDBDIF.getSeverity(), messageFDBCloud.getSeverity());
                    }
                    if (messageFDBDIF.getText() != messageFDBCloud.getText()) {
                        logDifference("text", messageFDBDIF.getText(), messageFDBCloud.getText());
                    }
                    if (messageFDBDIF.getType() != messageFDBCloud.getType()) {
                        logDifference("type", messageFDBDIF.getType(), messageFDBCloud.getType());
                    }
                }
                if (!foundMatch) {
                    Assert.fail("Didn't find a matching message in FDBDIF for FDBCloud record: "
                            + messageFDBCloud.getDrug().getGcnSeqNo());
                }
            }
        }
    }

    /**
     * Compares two drugsNotChecked lists. Fails test if the lists are not of
     * equal size with gcnseqnos from one list matching all those in the other
     * list. Logs field differences.
     * 
     * @param resultsFDBDIF
     * @param resultsFDBCloud
     */
    private void compareDrugsNotChecked(Collection<DrugCheckVo> drugsNotCheckedFDBDIF,
            Collection<DrugCheckVo> drugsNotCheckedFDBCloud) {
        if (drugsNotCheckedFDBDIF == null && drugsNotCheckedFDBCloud == null) {
            return;
        } else if (drugsNotCheckedFDBDIF == null && drugsNotCheckedFDBCloud != null) {
            LOG.error("FDBDIF does not have drugsNotChecked, but FDBCloud has drugsNotChecked.");
            return;
        } else if (drugsNotCheckedFDBDIF != null && drugsNotCheckedFDBCloud == null) {
            LOG.error("FDBDIF has drugsNotChecked, but FDBCloud does not have drugsNotChecked.");
            return;
        }
        Assert.assertEquals("drugsNotChecked Lists do not match.", drugsNotCheckedFDBDIF.size(),
                drugsNotCheckedFDBCloud.size());
        for (DrugCheckVo drugCheckVoFDBCloud : drugsNotCheckedFDBCloud) {
            for (DrugCheckVo drugCheckVoFDBDIF : drugsNotCheckedFDBDIF) {
                boolean foundMatch = false;
                if (drugCheckVoFDBDIF.getGcnSeqNo().equals(drugCheckVoFDBCloud.getGcnSeqNo())) {
                    foundMatch = true;
                    compareDrugCheck(drugCheckVoFDBCloud, drugCheckVoFDBDIF);
                }
                if (!foundMatch) {
                    Assert.fail("Didn't find a matching drugNotChecked in FDBDIF for FDBCloud record: "
                            + drugCheckVoFDBCloud.getGcnSeqNo());
                }
            }
        }
    }

    /**
     * @param drugCheckVoFDBCloud
     * @param drugCheckVoFDBDIF
     */
    private void compareDrugCheck(DrugCheckVo drugCheckVoFDBCloud, DrugCheckVo drugCheckVoFDBDIF) {
        LOG.trace("Comparing DrugsNotChecked with gcnseqno: " + drugCheckVoFDBDIF.getGcnSeqNo());

        if (drugCheckVoFDBDIF.isDoseCheck() != drugCheckVoFDBCloud.isDoseCheck()) {
            logDifference("doseCheck", drugCheckVoFDBDIF.isDoseCheck(), drugCheckVoFDBCloud.isDoseCheck());
        }
        if (drugCheckVoFDBDIF.isProspective() != drugCheckVoFDBCloud.isProspective()) {
            logDifference("doseCheck", drugCheckVoFDBDIF.isProspective(), drugCheckVoFDBCloud.isProspective());
        }
        if (drugCheckVoFDBDIF.getDoseRate() != drugCheckVoFDBCloud.getDoseRate()) {
            logDifference("doseRate", drugCheckVoFDBDIF.getDoseRate(), drugCheckVoFDBCloud.getDoseRate());
        }
        if (drugCheckVoFDBDIF.getDoseRoute() != drugCheckVoFDBCloud.getDoseRoute()) {
            logDifference("doseRate", drugCheckVoFDBDIF.getDoseRoute(), drugCheckVoFDBCloud.getDoseRoute());
        }
        if (drugCheckVoFDBDIF.getDoseType() != drugCheckVoFDBCloud.getDoseType()) {
            logDifference("doseType", drugCheckVoFDBDIF.getDoseType(), drugCheckVoFDBCloud.getDoseType());
        }
        if (drugCheckVoFDBDIF.getDoseUnit() != drugCheckVoFDBCloud.getDoseUnit()) {
            logDifference("doseUnit", drugCheckVoFDBDIF.getDoseUnit(), drugCheckVoFDBCloud.getDoseUnit());
        }
        if (drugCheckVoFDBDIF.getDrugName() != drugCheckVoFDBCloud.getDrugName()) {
            logDifference("drugName", drugCheckVoFDBDIF.getDrugName(), drugCheckVoFDBCloud.getDrugName());
        }
        if (drugCheckVoFDBDIF.getDuration() != drugCheckVoFDBCloud.getDuration()) {
            logDifference("duration", drugCheckVoFDBDIF.getDuration(), drugCheckVoFDBCloud.getDuration());
        }
        if (drugCheckVoFDBDIF.getDurationRate() != drugCheckVoFDBCloud.getDurationRate()) {
            logDifference("durationRate", drugCheckVoFDBDIF.getDurationRate(), drugCheckVoFDBCloud.getDurationRate());
        }
        if (drugCheckVoFDBDIF.getFrequency() != drugCheckVoFDBCloud.getFrequency()) {
            logDifference("frequency", drugCheckVoFDBDIF.getFrequency(), drugCheckVoFDBCloud.getFrequency());
        }
        if (drugCheckVoFDBDIF.getIen() != drugCheckVoFDBCloud.getIen()) {
            logDifference("IEN", drugCheckVoFDBDIF.getIen(), drugCheckVoFDBCloud.getIen());
        }
        if (drugCheckVoFDBDIF.getOrderNumber() != drugCheckVoFDBCloud.getOrderNumber()) {
            logDifference("orderNumber", drugCheckVoFDBDIF.getOrderNumber(), drugCheckVoFDBCloud.getOrderNumber());
        }
        if (drugCheckVoFDBDIF.getSingleDoseAmount() != drugCheckVoFDBCloud.getSingleDoseAmount()) {
            logDifference("singleDoseAmount", drugCheckVoFDBDIF.getSingleDoseAmount(),
                    drugCheckVoFDBCloud.getSingleDoseAmount());
        }
        if (drugCheckVoFDBDIF.getVuid() != drugCheckVoFDBCloud.getVuid()) {
            logDifference("VUID", drugCheckVoFDBDIF.getVuid(), drugCheckVoFDBCloud.getVuid());
        }
    }

    /**
     * @param drugChecksFDBDIF
     * @param drugChecksFDBCloud
     */
    private void compareDTCheckResults(Collection<DrugTherapyCheckResultVo> drugChecksFDBDIF,
            Collection<DrugTherapyCheckResultVo> drugChecksFDBCloud) {
        if (drugChecksFDBDIF == null && drugChecksFDBCloud == null) {
            return;
        } else if (drugChecksFDBDIF == null && drugChecksFDBCloud != null) {
            LOG.error("FDBDIF does not have DrugTherapyCheckResultVOs, but FDBCloud has DrugTherapyCheckResultVos.");
            return;
        } else if (drugChecksFDBDIF != null && drugChecksFDBCloud == null) {
            LOG.error("FDBDIF has DrugTherapyCheckResultVos, but FDBCloud does not have DrugTherapyCheckResultVos.");
            return;
        }

        for (DrugTherapyCheckResultVo dtCheckResultVoFDBCloud : drugChecksFDBCloud) {
            boolean foundMatch = false;
            for (DrugTherapyCheckResultVo dtCheckResultVoFDBDIF : drugChecksFDBDIF) {
                if (dtCheckResultVoFDBDIF.getId() == dtCheckResultVoFDBCloud.getId()) {
                    foundMatch = true;
                    System.out
                            .println("Comparing DrugTherapyCheckResultVo with id: " + dtCheckResultVoFDBCloud.getId());

                    if (dtCheckResultVoFDBDIF.isCustom() != dtCheckResultVoFDBCloud.isCustom()) {
                        logDifference("isCustom", dtCheckResultVoFDBDIF.isCustom(), dtCheckResultVoFDBCloud.isCustom());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getDuplicateClass(),
                            dtCheckResultVoFDBCloud.getDuplicateClass())) {
                        logDifference("duplicateClass", dtCheckResultVoFDBDIF.getDuplicateClass(),
                                dtCheckResultVoFDBCloud.getDuplicateClass());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getShortText(),
                            dtCheckResultVoFDBCloud.getShortText())) {
                        logDifference("shortText", dtCheckResultVoFDBDIF.getShortText(),
                                dtCheckResultVoFDBCloud.getShortText());
                    }

                    if (!ObjectUtils.equals(dtCheckResultVoFDBDIF.getAllowance(),
                            dtCheckResultVoFDBCloud.getAllowance())) {
                        logDifference("allowance", dtCheckResultVoFDBDIF.getAllowance(),
                                dtCheckResultVoFDBCloud.getAllowance());
                    }

                    Assert.assertEquals("Lists of DrugCheckVo do not match.", dtCheckResultVoFDBDIF.getDrugs(),
                            dtCheckResultVoFDBCloud.getDrugs());
                    break;
                }
            }
            if (!foundMatch) {
                Assert.fail("Didn't find a match in FDBDIF for FDBCloud record: " + dtCheckResultVoFDBCloud);
            }
        }
    }

    private void logDifference(String fieldName, Object fdbDIFValue, Object fdbCloudValue) {
        LOG.error("DIFF in " + fieldName + ":");
        LOG.error("FDBDIF   : " + fdbDIFValue);
        LOG.error("FDBCloud : " + fdbCloudValue);
    }
}